﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  06/05/13    Created
//
//---------------------------------------------------------------------------

namespace System.Windows.Forms
{
    using System.ComponentModel;
    using System.Windows.Forms;

    public static class ISynchronizeInvokeExtensions
    {
        /// <summary>
        /// Executes the specified delegate asynchronously on the thread the that the Control was created on.
        /// </summary>
        /// <param name="sender">The Control that is executing the specified delegate.</param>
        /// <param name="del">A delegate to a method that takes no parameters.</param>
        /// <returns>An System.IAsyncResult interface that represents the asynchronous operation started by calling this method.</returns>
        public static IAsyncResult BeginInvoke(this ISynchronizeInvoke sender, Action action)
        {
            if (sender == null)
                throw new ArgumentNullException("sender");

            return sender.BeginInvoke(action, null);
        }

        /// <summary>
        /// Asynchronously executes the delegate on the thread that created this object.
        /// </summary>
        /// <param name="sender">The Control that is executing the specified delegate.</param>
        /// <param name="func">A delegate to a method that takes no parameters and returns a value.</param>
        /// <returns>An System.IAsyncResult interface that represents the asynchronous operation started by calling this method.</returns>
        public static IAsyncResult BeginInvoke<T>(this ISynchronizeInvoke sender, Func<T> func)
        {
            if (sender == null)
                throw new ArgumentNullException("sender");

            return sender.BeginInvoke(func, null);
        }

        /// <summary>
        /// Executes the specified delegate on the thread the that the Control was created on, waits until completed and returns a value.
        /// </summary>
        /// <typeparam name="T">The type of value the method will return.</typeparam>
        /// <param name="sender">The Control that is executing the specified delegate.</param>
        /// <param name="func">A delegate to a method that takes no parameters and returns a value.</param>
        /// <returns>Value returned from method.</returns>
        public static T Invoke<T>(this ISynchronizeInvoke sender, Func<T> func)
        {
            if (sender == null)
                throw new ArgumentNullException("sender");

            return (T)sender.Invoke(func, null);
        }

        /// <summary>
        /// Executes the specified delegate on the thread the that the Control was created on and waits until completed.
        /// </summary>
        /// <typeparam name="T">The type of value the method will return.</typeparam>
        /// <param name="sender">The Control that is executing the specified delegate.</param>
        /// <param name="action">A delegate to a method that takes no parameters and does not return a value.</param>
        public static void Invoke(this ISynchronizeInvoke sender, Action action)
        {
            if (sender == null)
                throw new ArgumentNullException("sender");

            sender.Invoke(action, null);
        }
    }
}
